/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file Image.hpp
/// @desc A widget that contains and renders an image provided by JGL.
/// @edit 2024-08-05

#pragma once

#include <JUI/Base/Widget.hpp>
#include <JUI/Base/ImageBase.hpp>

namespace JUI
{
    class Image : public Widget, public ImageBase
    {
    public:
        Image();
        explicit Image(Widget* parent);
        explicit Image(Widget* parent, JGL::Texture* tex);
        ~Image() override {}

    public:
        Vector2 GetAbsoluteSize() const override;
        Vector2 GetAbsolutePosition() const override;

        void Draw() override;
        void Update(float delta) override;


        bool FitImageToParent() const;
        void FitImageToParent(bool on);

    protected:
        bool fit_image_to_parent;
    };
}