#include <JUI/Base/TextBase.hpp>
#include "JGL/JGL.h"


namespace JUI {
    void TextBase::SetContent(const std::string &content) { this->content = content; }

    void TextBase::SetTextColor(const Color4 &color) { this->text_color = color; }

    void TextBase::SetOutlineColor(const Color4 &color) { this->outline_color = color; }

    void TextBase::SetHorizontalTextAlign(const TextAlign::H &align) { this->h_align = align;}

    void TextBase::SetVerticalTextAlign(const TextAlign::V &align) { this->v_align = align; }

    void TextBase::SetTextAlign(const TextAlign::H &h_align, const TextAlign::V &v_align) {
        SetHorizontalTextAlign(h_align);
        SetVerticalTextAlign(v_align);
    }

    void TextBase::AlignLeft() { SetHorizontalTextAlign(TextAlign::H::Left);}

    void TextBase::AlignRight() { SetHorizontalTextAlign(TextAlign::H::Right);}

    void TextBase::AlignTop() {SetVerticalTextAlign(TextAlign::V::Top);}

    void TextBase::AlignBottom() {SetVerticalTextAlign(TextAlign::V::Bottom);}

    void TextBase::AlignCenterBoth() {
        SetVerticalTextAlign(TextAlign::V::Center);
        SetHorizontalTextAlign(TextAlign::H::Center);
    }

    void TextBase::Center() { AlignCenterBoth(); }

    void TextBase::AlignCenterHorizontally() {
        SetHorizontalTextAlign(TextAlign::H::Center);
    }

    void TextBase::AlignCenterVertically() {
        SetVerticalTextAlign(TextAlign::V::Center);
    }

    void TextBase::SetWordWrap(bool wrap) { word_wrap = wrap; }

    void TextBase::Draw(const Vector2 &abs_pos, const Vector2 &abs_size) {

        // Calculate how much to origin the text based on alignment.
        float align_x = abs_pos.x;
        float align_y = abs_pos.y;

        auto bounds = this->GetFont().MeasureString(this->content, this->text_size);

        if (h_align == TextAlign::H::Left) {
            // Render from left-side of boundary.
            align_x = abs_pos.x;
        }
        if (h_align == TextAlign::H::Center) {
            // Render from horizontal center, origin by half of the text's width.
            align_x = abs_pos.x + (abs_size.x / 2) - (bounds.x / 2);
        }
        if (h_align == TextAlign::H::Right) {
            // Render from right-side of boundary, origin by text width.
            align_x = abs_pos.x + abs_size.x - bounds.x;
        }
        if (v_align == TextAlign::V::Top) {
            // Render from top of boundary.
            align_y = abs_pos.y;
        }
        if (v_align == TextAlign::V::Center) {
            // Render from vertical center, origin by half of the text's height.
            align_y = abs_pos.y + (abs_size.y / 2) - (bounds.y / 2);
        }
        if (v_align == TextAlign::V::Bottom) {
            // Render from bottom of boundary, origin by the text's height.
            align_y = abs_pos.y + abs_size.y - bounds.y;
        }

        // Now we render.
        float scale = 1.f;

        JGL::J2D::Begin();
            JGL::J2D::DrawString(
                    this->text_color,
                    this->content,
                    align_x, align_y,
                    scale,
                    this->text_size,
                    this->set_font);
        JGL::J2D::End();

    }

    std::string TextBase::GetContent() const { return content;}

    Color4 TextBase::GetTextColor() const { return text_color;}

    Color4 TextBase::GetOutlineColor() const { return outline_color;}

    TextAlign::H TextBase::GetHorizontalTextAlign() const { return h_align;}

    TextAlign::V TextBase::GetVerticalTextAlign() const {return v_align;}

    Vector2 TextBase::GetTextBounds() const { return GetFont().MeasureString(content, text_size); }

    JGL::Font TextBase::GetFont() const { return set_font;}

    u32 TextBase::GetTextSize() const { return text_size; }

    void TextBase::SetFont(const JGL::Font &font) {
        set_font = font;
    }

    TextBase::TextBase() {}

    void TextBase::SetTextSize(u32 size) {
        text_size = size;
    }

    void TextBase::Update(float delta) {}
}



