#include "JGL/JGL.h"
#include "JUI/Widgets/Rect.hpp"
#include <jlog/Logger.hpp>

namespace JUI {
    Rect::Rect(): Widget(), RectBase() {}

    Rect::Rect(Widget *parent): Rect() {
        this->SetParent(parent);
    }

    void Rect::Draw() {
        if (!visible)
            return;

        J2D::Begin();
        Vector2 abs_pos = GetAbsolutePosition();
        Vector2 abs_size = GetAbsoluteSize();

        auto root_size = GetFamilyTreeRoot()->GetAbsoluteSize();

        GLint *old_scissor_bounds;
        bool clip_was_enabled;
        // TODO: Re-enable clipping
        if (clips_descendants) {
            clip_was_enabled = glIsEnabled(GL_SCISSOR_TEST);
            if (clip_was_enabled)
                glGetIntegerv(GL_SCISSOR_BOX, old_scissor_bounds);
            
            float presumed_screen_height = 600;
            glScissor(abs_pos.x, presumed_screen_height-abs_size.y-abs_pos.y, abs_size.x, abs_size.y);
            glEnable(GL_SCISSOR_TEST);
        }


        RectBase::Draw(abs_pos, abs_size);

        // Draw Child Elements with scissor clipping still active
        Widget::Draw();

        // Returns clip to previous state
        if (clips_descendants)
        {
            //glScissor(old_scissor_bounds[0], old_scissor_bounds[1], old_scissor_bounds[2], old_scissor_bounds[3]);

            if (!clip_was_enabled) {}
                glDisable(GL_SCISSOR_TEST);

        }

        J2D::End();
    }


    void Rect::Update(float delta) {
        if (IsMouseInside() && !mouse_inside_debounce)
        {
            MouseEnter.Invoke(last_known_mouse_pos);
            mouse_inside_debounce = true;
        }

        if (!IsMouseInside() && mouse_inside_debounce)
        {
            MouseExit.Invoke(last_known_mouse_pos);
            mouse_inside_debounce = false;
        }

        Widget::Update(delta);
    }

    bool Rect::IsMouseInside() const {
        float x = last_known_mouse_pos.x;
        float y = last_known_mouse_pos.y;
        auto pos = GetAbsolutePosition();
        auto size = GetAbsoluteSize();

        //DEBUG(std::format("X {} {} Y {} {}", x,pos.x, y,pos.y));

        if (x > pos.x && y > pos.y && x < pos.x + size.x && y < pos.y + size.y){
            return true;
        }
        return false;
    }

}


